/*****************************************************************************/
/*                                                                           */
/*  asteroids.java                                                           */
/*  Simple C# XNA Game                                                       */
/*  www.xbdev.net                                                            */
/*  bkenwright@xbdev.net                                                     */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*  Controls:                                                                */
/*                                                                           */
/*  Start            - Start Game    Pause           - Pause Game            */
/*  Controller Left  - Rotate Left   Controller Up   - Fire Thrusters        */
/*  Cursor Right     - Rotate Right  Controller Down - Fire Retro Thrusters  */
/*  A                - Fire Cannon   Y               - Hyperspace            */
/*                                                                           */
/*****************************************************************************/
/*
    Simple and Fun :)
 
    Author: bkenwright@xbdev.net
    Copyright: bkenwright@xbdev.net
  
    Should be very simple to follow whats going in with the code, uses basic
    line drawing for objects, probably the most complicated code to grasp 
    might be the collision detection stuff, but hopfully I've commeted it
    enough so that you can see whats going on.
*/
/*****************************************************************************/


using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

using System.Diagnostics; // So we can put some debug output


/*****************************************************************************/
/*                                                                           */
/* Debug Class                                                               */
/*                                                                           */
/* So I could keep all my prints and halts in one place I created my own     */
/* debug print class which I can enable and disable.                         */
/*                                                                           */
/*****************************************************************************/

class Debug
{
    public static void Print(string str)   { System.Diagnostics.Debug.Write(str);     }
    public static void PrintLn(string str) { System.Diagnostics.Debug.WriteLine(str); }
    public static bool debugOn = true;
    public static bool allowUpdates = true;

}


/*****************************************************************************/
/*                                                                           */
/* MyDraw Class                                                              */
/*                                                                           */
/* All drawing to the screen all goes through this one class, so I could     */
/* easily implement the drawing system.                                      */
/*                                                                           */
/*****************************************************************************/
public class myGraphics
{
    GraphicsDevice graphicsDevice;
    VertexPositionColor[] verts;
    SpriteBatch spriteBatch;
    public SpriteFont spriteFont;
    Color colour;

    public int SAFE_SCREEN_LEFT   = 0;
    public int SAFE_SCREEN_TOP    = 0;
    public int SAFE_SCREEN_RIGHT  = 0;
    public int SAFE_SCREEN_BOTTOM = 0;

    public myGraphics(GraphicsDevice graphicsDevice, SpriteFont spriteFont)
    {
        this.graphicsDevice = graphicsDevice;
        this.spriteFont = spriteFont;
        this.spriteBatch = new SpriteBatch(graphicsDevice);
        verts = new VertexPositionColor[2];

        Vector2 screenSize = 
        new Vector2( graphicsDevice.Viewport.Width,
                     graphicsDevice.Viewport.Height);
        
        // Assume Safe Screen is 10% less than full screen
        Vector2 delta = (screenSize/100.0f)*10.0f;

        SAFE_SCREEN_LEFT   = (int)delta.X;
        SAFE_SCREEN_TOP    = (int)delta.Y;
        SAFE_SCREEN_RIGHT  = (int)(screenSize.X - delta.X);
        SAFE_SCREEN_BOTTOM = (int)(screenSize.Y - delta.Y);
    }

    public void setColor(Color col)
    {
        colour = col;
    }

    public Color getColor()
    {
        return colour;
    }

    public void drawPolyline(int[] xpoints,
                             int[] ypoints,
                             int numPoints)
    {
        if (numPoints <= 0) return;

        for (int i = 0; i < numPoints; i++)
        {
            drawLine(xpoints[i], ypoints[i], xpoints[(i+1)%numPoints], ypoints[(i+1)%numPoints]);
        }
    }

    public void fillPolygon(int[] xpoints,
                            int[] ypoints,
                            int numPoints)
    {
    }

    public void fillRect(int x, int y, int w, int h)
    {
    }

    static float xs = 0.06f;
    static float ys = 0.07f;
    static float xoff = -400.0f;
    static float yoff = -300.0f;

    public void drawLine(int x0, int y0, int x1, int y1)
    {
        int ww = graphicsDevice.Viewport.Width;
        int hh = graphicsDevice.Viewport.Height;

        float x00 = (((float)x0 + xoff) * xs);
        float x11 = (((float)x1 + xoff) * xs);
        float y00 = (((float)y0 + yoff) * ys);
        float y11 = (((float)y1 + yoff) * ys);

        Color ccc = new Color(colour.R / 255.0f, colour.G / 255.0f, colour.B / 255.0f);

        verts[0].Position = new Vector3(x00, y00, 0);
        verts[0].Color = ccc;
        verts[1].Position = new Vector3(x11, y11, 0);
        verts[1].Color = ccc;

        

        graphicsDevice.DrawUserPrimitives<VertexPositionColor>(PrimitiveType.LineList, verts, 0, 1);
    }

    public void drawString(string str, int x, int y)
    {
        graphicsDevice.RenderState.DepthBufferEnable = true;
        spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
        spriteBatch.DrawString(spriteFont,          // spriteFont
                     str,                           // String
                     new Vector2(x, y),             // ScreenPos
                     colour);                       // Colour
        spriteBatch.End();
    }

    public void DrawLine2D(Vector2 from, Vector2 to, Color colour)
    {
        VertexPositionColor[]
        pnts = {new VertexPositionColor(new Vector3(from.X, from.Y, 0), colour),
                new VertexPositionColor(new Vector3(to.X,   to.Y,   0), colour)};

        VertexDeclaration
        basicEffectVertexDeclaration 
        = new VertexDeclaration(graphicsDevice, VertexPositionColor.VertexElements);

        BasicEffect basicEffect;
        basicEffect = new BasicEffect(graphicsDevice, null);
        basicEffect.Alpha = 1.0f;
        basicEffect.VertexColorEnabled = true;
        basicEffect.TextureEnabled = false;
        basicEffect.Projection = Matrix.CreateOrthographicOffCenter(0.0F,
                                            graphicsDevice.Viewport.Width,
                                            graphicsDevice.Viewport.Height, 0.0f,
                                            0.0F, -1.0F);

        basicEffect.View = Matrix.Identity;
        basicEffect.World = Matrix.Identity;
        graphicsDevice.VertexDeclaration        = basicEffectVertexDeclaration;
        graphicsDevice.RenderState.PointSize    = 2; // Line Thickness
        graphicsDevice.RenderState.FillMode     = FillMode.Solid;
        graphicsDevice.RenderState.CullMode     = CullMode.None;
        graphicsDevice.VertexDeclaration = basicEffectVertexDeclaration;
        basicEffect.Begin();
        foreach (EffectPass pass in basicEffect.CurrentTechnique.Passes)
        {
            pass.Begin();
            graphicsDevice.DrawUserPrimitives<VertexPositionColor>(
                        PrimitiveType.LineList,
                        pnts, 0, 1);

            pass.End();
        }
        basicEffect.End();
    }// End DrawLine2D(..)

    public void DrawSafeScreen()
    {
        Vector2 screenSize = 
        new Vector2( graphicsDevice.Viewport.Width,
                     graphicsDevice.Viewport.Height);
        // Assume Safe Screen is 10% less than full screen
        Vector2 delta = (screenSize/100.0f)*10.0f;

        DrawLine2D(delta,                                                   new Vector2(screenSize.X-delta.X,delta.Y),                  Color.Blue);
        DrawLine2D(new Vector2(screenSize.X-delta.X,delta.Y),               new Vector2(screenSize.X-delta.X,screenSize.Y-delta.Y),     Color.Blue);
        DrawLine2D(new Vector2(screenSize.X-delta.X,screenSize.Y-delta.Y),  new Vector2(delta.X,screenSize.Y-delta.Y),                  Color.Blue);
        DrawLine2D(new Vector2(delta.X,screenSize.Y-delta.Y),               delta,                                                      Color.Blue);
    }


};



/*****************************************************************************/
/*                                                                           */
/* myShape class                                                             */
/*                                                                           */
/* Container class holds a set of coords wich make up the shape.  Array of   */
/* x and y values.  These values form an enclosed shape - we can use this    */
/* information in some of the member functions to do collion detection.!     */
/*                                                                           */
/*****************************************************************************/
public class myShape
{
	static int MAX_NUM_POINTS = 100;   // Maximum size of our array

	public int[] xpoints;              // Array of points
	public int[] ypoints;
	public int npoints;                // Total number of points

	// Default Constructor
	public myShape()
	{
		xpoints = new int[MAX_NUM_POINTS];
		ypoints = new int[MAX_NUM_POINTS];
		npoints = 0;
	}

	public void addPoint(int x, int y)
	{
		xpoints[npoints] = x;
		ypoints[npoints] = y;
		npoints++;
	}

    public Boolean contains(int x, int y)
	{
		Boolean bInside = false;
		
		bInside = insidePoly(this.xpoints, this.ypoints, this.npoints,
			                  x, y);

		return bInside;
	}


    public Boolean insidePoly( int[] xcoords,
                               int[] ycoords,
                               int num_coords,
                               int x, int y)
    {
        if (num_coords < 2) return false;

        bool inside = false;
        for (int i=0, j=num_coords-1; i<num_coords; j=i, i++)
        {
            Vector2 u0 = new Vector2(xcoords[i], ycoords[i]);
            Vector2 u1 = new Vector2(xcoords[j], ycoords[j]);

            if (y < u1.Y)
            {
                if (u0.Y<=y)
                {
                    if ((y-u0.Y)*(u1.X-u0.X) > (x-u0.X)*(u1.Y-u0.Y))
                        inside = !inside;
                }
            }
            else if (y<u0.Y)
            {
                if ((y-u0.Y)*(u1.X-u0.X) < (x-u0.X)*(u1.Y-u0.Y))
                    inside = !inside;
            }
        }
        return inside;
    }

    // Simple function - you pass an array of coords and a point, and it
    // determines if the point is inside the object.  The coords must 
    // form a closed object.
    /*
	public Boolean insidePoly( int[] xcoords, int[] ycoords, int num_coords,
		                int x0, int y0 )
	{
		// Line far away that we can use to reference our point to.
		double px = 2000;
		double py = 2000;

		int iLen = num_coords;

        // Need to close the shape, first and last points
		int totalIntersections = 0;
		for(int i=0; i<iLen; i+=1)        
		{
            double p0x = xcoords[i];
            double p0y = ycoords[i];

            double p1x = xcoords[(i+1) % iLen];
            double p1y = ycoords[(i+1) % iLen];


            bool hitTestA =
            IntersectLines2D(p0x, p0y, p1x, p1y,
                             px, py, x0, y0) > 0;

            bool hitTestB =
            IntersectLines2D(p0x, p0y, p1x, p1y,
                             -3333, 1323, x0, y0) > 0;

            if (hitTestA && hitTestB)
            {
                totalIntersections++;
            }

		}

		if( (totalIntersections % 2)==0 ) // even
			return true;
		else
			return false;
	}// End insidePoly(..)

	// Used by insidePoly(..), it return true or false if the two lines
	// cross each other.
	int IntersectLines2D(   double p0x, double p0y, double p1x, double p1y,
		                    double p2x, double p2y, double p3x, double p3y)
	{
		// Uses parametric lines. (e.g. p = p0 + (p1-p0)*t ) where t is from 0 to 1;
		double p0vx  = p1x - p0x;
		double p0vy  = p1y - p0y;

		double p1vx  = p3x - p2x;
		double p1vy  = p3y - p2y;

		// Test -1- Check for parallel ines.  If the direction vectors are scalar
		// multiples then the lines are parallel and can't possibly intersect unless
		// the lines overlap
		double det_p0p1 = (p0vx*p1vy - p0vy*p1vx);
		if( Math.Abs( det_p0p1) <= 0.001 )
			return 0; // Parallel lines - hence no intersection

		// Test -2- Computer the t0 and t1 values for intersection, where we have
		// two lines of hte form
		// p = p0 + v*t
		// p1 = p10 + v1*t1

		// p1.x = p10.x + v1.x*t1
		// p1.y = p10.y + v1.y*t1

		// p2 = p20 + v2*t2

		// p2.x = p20.x + v2.x*t2
		// p2.y = p20.y + v2.y*t2
		// Solve the system when x1 = x2 and y1 = y2

        
		double t0 = (p1vx*(p0y - p2y) - p1vy*(p0x - p2x) ) / det_p0p1;
		double t1 = (p0vx*(p0y - p2y) - p0vy*(p0x - p2x) ) / det_p0p1;


        if ((t0 > 0.0) && (t0 < 1.0) && (t1 >= 0.0) && (t1 <= 1.0))
			return 1; // We have an intersection
		else
			return 0; // Intersects out of bounds.

	}//End IntersectLines2D(..)
    */

}//End Class myShape



/*****************************************************************************/
/*                                                                           */
/* AsteroidObject class                                                      */
/*                                                                           */
/* Its so much easier to keep things together, so each object, rocks, our    */
/* ship, its bullets, alien ships are all contained in a class...this        */
/* class.  Tidy eh.  We can do collision detection and rendering all here    */
/* Movement updates and rotations.                                           */
/*                                                                           */
/*****************************************************************************/
public class AsteroidObject
{
    // Fields:
    public static int width  = 0;     // Dimensions of the graphics area.
    public static int height = 0;

    public myShape shape;             // Base sprite shape, centered at the origin (0,0).
    public Boolean active = false;    // Active flag.
    public double angle;              // Current angle of rotation.
    public double deltaAngle;         // Amount to change the rotation angle.
    public double x, y;               // Current position on screen.
    public double deltaX, deltaY;     // Amount to change the screen position.
    public myShape sprite;            // Final location and shape of sprite after
    // applying rotation and translation to get screen
    // position. Used for drawing on the screen and in
    // detecting collisions.

    // Constructors:
    public AsteroidObject()
    {
        this.shape = new myShape();
        this.active = false;
        this.angle = 0.0;
        this.deltaAngle = 0.0;
        this.x = 0.0;
        this.y = 0.0;
        this.deltaX = 0.0;
        this.deltaY = 0.0;
        this.sprite = new myShape();
    }

    // Methods:

    public Boolean advance()
    {
        Boolean wrapped;

        // Update the rotation and position of the sprite based on the delta
        // values. If the sprite moves off the edge of the screen, it is wrapped
        // around to the other side and TRUE is returnd.

        this.angle += this.deltaAngle;
        if (this.angle < 0)
            this.angle += 2 * Math.PI;
        if (this.angle > 2 * Math.PI)
            this.angle -= 2 * Math.PI;
        wrapped = false;

        this.x += this.deltaX;
        this.y -= this.deltaY;
        
        if (this.x < -width / 2)
        {
            this.x += width;
            wrapped = true;
        }
        if (this.x > width / 2)
        {
            this.x -= width;
            wrapped = true;
        }
        
        if (this.y < -height / 2)
        {
            this.y += height;
            wrapped = true;
        }
        if (this.y > height / 2)
        {
            this.y -= height;
            wrapped = true;
        }

        return wrapped;
    }//End advance(..)

    public void render()
    {
        int i;

        // Render the sprite's shape and location by rotating it's base shape and
        // moving it to it's proper screen position.
        this.sprite = new myShape();
        for (i = 0; i < this.shape.npoints; i++)
            this.sprite.addPoint((int)Math.Round(this.shape.xpoints[i] * Math.Cos(this.angle) + this.shape.ypoints[i] * Math.Sin(this.angle)) + (int)Math.Round(this.x) + width / 2,
                                 (int)Math.Round(this.shape.ypoints[i] * Math.Cos(this.angle) - this.shape.xpoints[i] * Math.Sin(this.angle)) + (int)Math.Round(this.y) + height / 2);
    }//End render(..)

    public Boolean isColliding(AsteroidObject s)
    {
        int i;

        // Determine if one sprite overlaps with another, i.e., if any vertice
        // of one sprite lands inside the other.

        for (i = 0; i < s.sprite.npoints; i++)
            if (this.sprite.contains(s.sprite.xpoints[i], s.sprite.ypoints[i]))
                return true;
        for (i = 0; i < this.sprite.npoints; i++)
            if (s.sprite.contains(this.sprite.xpoints[i], this.sprite.ypoints[i]))
                return true;
        return false;


        //for (i = 0; i < s.sprite.npoints; i++)
        //	if (this.sprite.contains(s.sprite.xpoints[i], s.sprite.ypoints[i]))
        //		return true;
        //return false;


    }//End isColliding(..)

    // DEBUG FUNCTION!
    public void RenderisCollision(myGraphics offGraphics, AsteroidObject s)
    {
        for (int i = 0; i < s.sprite.npoints; i++)
            if (this.sprite.contains(s.sprite.xpoints[i], s.sprite.ypoints[i]))
            {
                // Draws a sequence of connected lines defined by arrays of x and y coordinates.
                offGraphics.drawPolyline(s.sprite.xpoints, // int[] x points
                                         s.sprite.ypoints, // int[] y points
                                         s.sprite.npoints);// number points
            }
    }

    public void drawPolyline(myGraphics offGraphics, Color col)
    {
        offGraphics.setColor(col);

        // Draws a sequence of connected lines defined by arrays of x and y coordinates.
        offGraphics.drawPolyline(this.sprite.xpoints, // int[] x points
                                 this.sprite.ypoints, // int[] y points
                                 this.sprite.npoints);// number points
    }

    public void fillPolygon(myGraphics offGraphics, Color col)
    {
        if (col != null)
            offGraphics.setColor(col);

        offGraphics.fillPolygon(this.sprite.xpoints, // int[] x points
                                this.sprite.ypoints, // int[] y points
                                this.sprite.npoints);// number points
    }

}//End class AsteroidObject






/*****************************************************************************/
/*                                                                           */
/* asteroids class                                                           */
/*                                                                           */
/* Entry point class - where our game body is located and game logic.        */
/* All our main code is in here.                                             */
/*                                                                           */
/*****************************************************************************/

public class asteroids : Microsoft.Xna.Framework.Game
{
    // Strings
	const String szName = "Asteroids";
	const String szVers = "Can you rise to the challenge?";

	// Thread control variables.
	//Thread loopThread;

	// Constants
	static int DELAY = 20;             // Milliseconds between screen and
	static int FPS   =                 // the resulting frame rate.
        (int)Math.Round(1000.0 / DELAY);

	static int MAX_SHOTS =  8;          // Maximum number of sprites
	static int MAX_ROCKS =  8;          // for photons, asteroids and
	static int MAX_SCRAP = 40;          // explosions.

	static int SCRAP_COUNT  = 2 * FPS;  // Timer counter starting values
	static int HYPER_COUNT  = 3 * FPS;  // calculated using number of
	static int MISSLE_COUNT = 4 * FPS;  // seconds x frames per second.
	static int STORM_PAUSE  = 2 * FPS;

	static int    MIN_ROCK_SIDES =   6; // Ranges for asteroid shape, size
	static int    MAX_ROCK_SIDES =  16; // speed and rotation.
	static int    MIN_ROCK_SIZE  =  20;
	static int    MAX_ROCK_SIZE  =  40;
	static double MIN_ROCK_SPEED =  40.0 / FPS;
	static double MAX_ROCK_SPEED = 240.0 / FPS;
	static double MAX_ROCK_SPIN  = Math.PI / FPS;

	static int MAX_SHIPS = 3;           // Starting number of ships for
												// each game.
	static int UFO_PASSES = 3;          // Number of passes for flying
												// saucer per appearance.

	// Ship's rotation and acceleration rates and maximum speed.
	static double SHIP_ANGLE_STEP = Math.PI / FPS;
	static double SHIP_SPEED_STEP = 15.0 / FPS;
	static double MAX_SHIP_SPEED  = 1.25 * MAX_ROCK_SPEED;

	static float FIRE_DELAY = 100.0f;   // Minimum number of milliseconds
										// required between photon shots.

    static Boolean SHIP_DIR_CONTROL = true;
    static Vector2 shipDir = new Vector2(0, 0);

	// Probablility of flying saucer firing a missle during any given frame
	// (other conditions must be met).
	static double MISSLE_PROBABILITY = 0.45 / FPS;

	static int BIG_POINTS    =  25;     // Points scored for shooting
	static int SMALL_POINTS  =  50;     // various objects.
	static int UFO_POINTS    = 250;
	static int MISSLE_POINTS = 500;

	// Number of points the must be scored to earn a new ship or to cause the
	// flying saucer to appear.
	static int NEW_SHIP_POINTS = 5000;
	static int NEW_UFO_POINTS  = 750;


	// Background stars.
	int     numStars;
	Point[] stars;

	// Game data.
	int score;
	int highScore;
	int newShipScore;
	int newUfoScore;


	// Flags for game state and options.
	//boolean loaded = false;
	Boolean paused;
    int pauseDelay = 0;
	Boolean playing;
    Boolean detail;

	// Key flags.
    Boolean left = false;
    Boolean right = false;
    Boolean up = false;
    Boolean down = false;


	// Sprite objects.
	AsteroidObject   ship;
	AsteroidObject   fwdThruster, revThruster;
	AsteroidObject   ufo;
	AsteroidObject   missle;
	AsteroidObject[] photons    = new AsteroidObject[MAX_SHOTS];
	AsteroidObject[] asteroidss = new AsteroidObject[MAX_ROCKS];
	AsteroidObject[] explosions = new AsteroidObject[MAX_SCRAP];


	// Ship data.
	int shipsLeft;       // Number of ships left in game, including current one.
	int shipCounter;     // Timer counter for ship explosion.
	int hyperCounter;    // Timer counter for hyperspace.


	// Photon data.
	int   photonIndex;    // Index to next available photon sprite.
	long  photonTime;     // Time value used to keep firing rate constant.


	// Flying saucer data.
	int ufoPassesLeft;    // Counter for number of flying saucer passes.
	int ufoCounter;       // Timer counter used to track each flying saucer pass.


	// Missle data.
	int missleCounter;    // Counter for life of missle.


	// Asteroid data.
	Boolean[] asteroidIsSmall = new Boolean[MAX_ROCKS];    // Asteroid size flag.
	int       asteroidsCounter;                            // Break-time counter.
	double    asteroidsSpeed;                              // Asteroid speed.
	int       asteroidsLeft;                               // Number of active asteroids.


	// Explosion data.
	int[] explosionCounter = new int[MAX_SCRAP];  // Time counters for explosions.
	int   explosionIndex;                         // Next available explosion sprite.


	// Data for the screen font.
    int gameOverDisplayDelay = 10000;
    int vibrateDelay = 10000;

    // Graphics Device so we can draw with the xna
    GraphicsDeviceManager graphics;
    VertexDeclaration myVertexDeclaration;

    myGraphics offGraphics;
    BasicEffect effect;
    GameTime gameTime = new GameTime();

    // Simple Game Sounds
    AudioEngine audioEngine;
    WaveBank waveBank;
    SoundBank soundBank;

    bool constrollerDisconnected = false;


    public asteroids()
    {
        graphics = new GraphicsDeviceManager(this);
        Content.RootDirectory = "Content";

        // So we can check the Guide.IsVisible flag
        Components.Add(new GamerServicesComponent(this));
    }

    Random random = new System.Random();

    public void init()
    {
        /*
        Dimension d = getSize();
        // Set up key event handling and set focus to applet window.
        addKeyListener(this);
        requestFocus();
        */

        int i;
        // Save the screen size.

        AsteroidObject.width = graphics.GraphicsDevice.Viewport.Width;
        AsteroidObject.height = graphics.GraphicsDevice.Viewport.Height;

        // Generate the starry background.
        numStars = AsteroidObject.width * AsteroidObject.height / 5000;
        stars = new Point[numStars];
        for (i = 0; i < numStars; i++)
            stars[i] = new Point((int)(random.NextDouble() * AsteroidObject.width), (int)(random.NextDouble() * AsteroidObject.height));

        // Create shape for the ship sprite.
        ship = new AsteroidObject();
        ship.shape.addPoint(0, -10);
        ship.shape.addPoint(7, 10);
        ship.shape.addPoint(-7, 10);

        // Create shapes for the ship thrusters.
        fwdThruster = new AsteroidObject();
        fwdThruster.shape.addPoint(0, 12);
        fwdThruster.shape.addPoint(-3, 16);
        fwdThruster.shape.addPoint(0, 26);
        fwdThruster.shape.addPoint(3, 16);
        revThruster = new AsteroidObject();
        revThruster.shape.addPoint(-2, 12);
        revThruster.shape.addPoint(-4, 14);
        revThruster.shape.addPoint(-2, 20);
        revThruster.shape.addPoint(0, 14);
        revThruster.shape.addPoint(2, 12);
        revThruster.shape.addPoint(4, 14);
        revThruster.shape.addPoint(2, 20);
        revThruster.shape.addPoint(0, 14);

        // Create shape for each photon sprites.
        for (i = 0; i < MAX_SHOTS; i++)
        {
            photons[i] = new AsteroidObject();
            photons[i].shape.addPoint(1, 1);
            photons[i].shape.addPoint(1, -1);
            photons[i].shape.addPoint(-1, 1);
            photons[i].shape.addPoint(-1, -1);
        }

        // Create shape for the flying saucer.
        ufo = new AsteroidObject();
        ufo.shape.addPoint(-15, 0);
        ufo.shape.addPoint(-10, -5);
        ufo.shape.addPoint(-5, -5);
        ufo.shape.addPoint(-5, -8);
        ufo.shape.addPoint(5, -8);
        ufo.shape.addPoint(5, -5);
        ufo.shape.addPoint(10, -5);
        ufo.shape.addPoint(15, 0);
        ufo.shape.addPoint(10, 5);
        ufo.shape.addPoint(-10, 5);

        // Create shape for the guided missle.
        missle = new AsteroidObject();
        missle.shape.addPoint(0, -4);
        missle.shape.addPoint(1, -3);
        missle.shape.addPoint(1, 3);
        missle.shape.addPoint(2, 4);
        missle.shape.addPoint(-2, 4);
        missle.shape.addPoint(-1, 3);
        missle.shape.addPoint(-1, -3);

        // Create asteroid sprites.
        for (i = 0; i < MAX_ROCKS; i++)
            asteroidss[i] = new AsteroidObject();

        // Create explosion sprites.

        for (i = 0; i < MAX_SCRAP; i++)
            explosions[i] = new AsteroidObject();

        // Initialize game data and put us in 'game over' mode.
        highScore = 0;
        detail = true;
        initGame();
        endGame();
    }

    public void initGame()
    {
        // Initialize game data and sprites.

        score = 0;
        shipsLeft = MAX_SHIPS;
        asteroidsSpeed = MIN_ROCK_SPEED;
        newShipScore = NEW_SHIP_POINTS;
        newUfoScore = NEW_UFO_POINTS;
        initShip();
        initPhotons();
        stopUfo();
        stopMissle();
        initAsteroids();
        initExplosions();
        playing = true;
        paused = false;
        photonTime = 0;
    }

    public void endGame()
    {
        // Stop ship, flying saucer, guided missle and associated sounds.

        playing = false;
        stopShip();
        stopUfo();
        stopMissle();
    }

    // This is the main loop.
    public void run()
    {
        if (!paused)
        {
            // Move and process all sprites.
            updateShip();
            updatePhotons();
            updateUfo();
            updateMissle();
            updateAsteroids();
            updateExplosions();

            // Check the score and advance high score, add a new ship or start the
            // flying saucer as necessary.

            if (score > highScore)
                highScore = score;
            if (score > newShipScore)
            {
                newShipScore += NEW_SHIP_POINTS;
                shipsLeft++;
            }
            if (playing && score > newUfoScore && !ufo.active)
            {
                newUfoScore += NEW_UFO_POINTS;
                ufoPassesLeft = UFO_PASSES;
                initUfo();
            }

            // If all asteroids have been destroyed create a new batch.

            if (asteroidsLeft <= 0)
                if (--asteroidsCounter <= 0)
                    initAsteroids();
        }

        vibrateDelay++;
        if (GamePad.GetState(PlayerIndex.One).IsConnected)
        {
            if (vibrateDelay < 50)
            {
                float val = 50.0f / (50.0f - (float)vibrateDelay*2.0f);
                if (val < 0) val = 0.0f;
                GamePad.SetVibration(PlayerIndex.One, val, 0.1f);
            }
            else
            {
                GamePad.SetVibration(PlayerIndex.One, 0.0f, 0.0f);
            }
        }
        if (vibrateDelay > 1000) vibrateDelay = 1000;
    }


    public void initShip()
    {

        // Reset the ship sprite at the center of the screen.
        ship.active = true;
        ship.angle = 0.0;
        ship.deltaAngle = 0.0;
        ship.x = 0.0;
        ship.y = 0.0;
        ship.deltaX = 0.0;
        ship.deltaY = 0.0;
        ship.render();

        // Initialize thruster sprites.
        fwdThruster.x = ship.x;
        fwdThruster.y = ship.y;
        fwdThruster.angle = ship.angle;
        fwdThruster.render();
        revThruster.x = ship.x;
        revThruster.y = ship.y;
        revThruster.angle = ship.angle;
        revThruster.render();

        hyperCounter = 0;
    }

    public void updateShip()
    {
    
        double dx, dy, speed;

        if (!playing)
            return;

        // Rotate the ship if left or right cursor key is down.

        shipDir.X = Math.Max(-1.0f, Math.Min(shipDir.X, 1.0f));
        shipDir.Y = Math.Max(-1.0f, Math.Min(shipDir.Y, 1.0f));

        if (SHIP_DIR_CONTROL)
        {
            const float contThresh = 0.001f;
            if ( Math.Abs(shipDir.X) >contThresh || Math.Abs(shipDir.Y) > contThresh)
            {
               ship.angle = Math.Atan2(-shipDir.X, shipDir.Y);
            }

            ship.deltaX += shipDir.X*0.2f;
            ship.deltaY += shipDir.Y*0.2f;
            speed = Math.Sqrt(ship.deltaX * ship.deltaX + ship.deltaY * ship.deltaY);
            if (speed > MAX_SHIP_SPEED)
            {
                ship.deltaX = ship.deltaY = 0;
            }

            ship.deltaAngle = 0.0f;
            ship.deltaX *= 0.95f;
            ship.deltaY *= 0.95f;
        }
        else
        {
            if (left)
            {
                ship.angle += SHIP_ANGLE_STEP;
                if (ship.angle > 2 * Math.PI)
                    ship.angle -= 2 * Math.PI;
            }
            if (right)
            {
                ship.angle -= SHIP_ANGLE_STEP;
                if (ship.angle < 0)
                    ship.angle += 2 * Math.PI;
            }

            // Fire thrusters if up or down cursor key is down.

            dx = SHIP_SPEED_STEP * -Math.Sin(ship.angle);
            dy = SHIP_SPEED_STEP * Math.Cos(ship.angle);
            if (up)
            {
                ship.deltaX += dx;
                ship.deltaY += dy;
            }
            if (down)
            {
                ship.deltaX -= dx;
                ship.deltaY -= dy;
            }
        }

        // Don't let ship go past the speed limit.
        if (up || down)
        {
            speed = Math.Sqrt(ship.deltaX * ship.deltaX + ship.deltaY * ship.deltaY);
            if (speed > MAX_SHIP_SPEED)
            {
                dx = MAX_SHIP_SPEED * -Math.Sin(ship.angle);
                dy = MAX_SHIP_SPEED * Math.Cos(ship.angle);
                if (up)
                    ship.deltaX = dx;
                else
                    ship.deltaX = -dx;
                if (up)
                    ship.deltaY = dy;
                else
                    ship.deltaY = -dy;
            }
        }


        // Move the ship. If it is currently in hyperspace, advance the countdown.
        if (ship.active)
        {
            ship.advance();
            ship.render();
            if (hyperCounter > 0)
                hyperCounter--;

            // Update the thruster sprites to match the ship sprite.
            fwdThruster.x = ship.x;
            fwdThruster.y = ship.y;
            fwdThruster.angle = ship.angle;
            fwdThruster.render();
            revThruster.x = ship.x;
            revThruster.y = ship.y;
            revThruster.angle = ship.angle;
            revThruster.render();
        }



            // Ship is exploding, advance the countdown or create a new ship if it is
        // done exploding. The new ship is added as though it were in hyperspace.
        // (This gives the player time to move the ship if it is in imminent
        // danger.) If that was the last ship, end the game.

        else
        {
            if (--shipCounter <= 0)
                if (shipsLeft > 0)
                {
                    initShip();
                    hyperCounter = HYPER_COUNT;
                }
                else
                    endGame();
        }
    }

    public void stopShip()
    {
        ship.active = false;
        shipCounter = SCRAP_COUNT;
        if (shipsLeft > 0)
            shipsLeft--;
    }

    public void initPhotons()
    {
        for (int i = 0; i < MAX_SHOTS; i++)
            photons[i].active = false;
        photonIndex = 0;
    }

    public void updatePhotons()
    {
        // Move any active photons. Stop it when its counter has expired.

        for (int i = 0; i < MAX_SHOTS; i++)
            if (photons[i].active)
            {
                if (!photons[i].advance())
                    photons[i].render();
                else
                    photons[i].active = false;
            }
    }

    public void initUfo()
    {
        double angle, speed;

        // Randomly set flying saucer at left or right edge of the screen.

        ufo.active = true;
        ufo.x = -AsteroidObject.width / 2;
        ufo.y = random.NextDouble() * 2 * AsteroidObject.height - AsteroidObject.height;
        angle = random.NextDouble() * Math.PI / 4 - Math.PI / 2;
        speed = MAX_ROCK_SPEED / 2 + random.NextDouble() * (MAX_ROCK_SPEED / 2);
        ufo.deltaX = speed * -Math.Sin(angle);
        ufo.deltaY = speed * Math.Cos(angle);
        if (random.NextDouble() < 0.5)
        {
            ufo.x = AsteroidObject.width / 2;
            ufo.deltaX = -ufo.deltaX;
        }
        if (ufo.y > 0)
            ufo.deltaY = 0;
        ufo.render();

        ufoCounter = (int)Math.Abs(AsteroidObject.width / ufo.deltaX);
    }

    public void updateUfo()
    {
        int i, d;

        // Move the flying saucer and check for collision with a photon. Stop it
        // when its counter has expired.

        if (ufo.active)
        {
            if (--ufoCounter <= 0)
            {
                if (--ufoPassesLeft > 0)
                    initUfo();
                else
                    stopUfo();
            }
            if (ufo.active)
            {
                ufo.advance();
                ufo.render();
                for (i = 0; i < MAX_SHOTS; i++)
                    if (photons[i].active && ufo.isColliding(photons[i]))
                    {
                        //if (sound)
                        //  crashSound.play();
                        explode(ufo);
                        stopUfo();
                        score += UFO_POINTS;
                    }

                // On occassion, fire a missle at the ship if the saucer is not too
                // close to it.
                d = (int)Math.Max(Math.Abs(ufo.x - ship.x), Math.Abs(ufo.y - ship.y));
                if (ship.active && hyperCounter <= 0 &&
                    ufo.active && !missle.active &&
                    d > MAX_ROCK_SPEED * FPS / 2 &&
                    random.NextDouble() < MISSLE_PROBABILITY)
                    initMissle();
            }
        }
    }

    public void stopUfo()
    {
        ufo.active = false;
        ufoCounter = 0;
        ufoPassesLeft = 0;
    }

    public void initMissle()
    {
        missle.active = true;
        missle.angle = 0.0;
        missle.deltaAngle = 0.0;
        missle.x = ufo.x;
        missle.y = ufo.y;
        missle.deltaX = 0.0;
        missle.deltaY = 0.0;
        missle.render();
        missleCounter = MISSLE_COUNT;
    }

    public void updateMissle()
    {
        int i;

        // Move the guided missle and check for collision with ship or photon. Stop
        // it when its counter has expired.

        if (missle.active)
        {
            if (--missleCounter <= 0)
                stopMissle();
            else
            {
                guideMissle();
                missle.advance();
                missle.render();
                for (i = 0; i < MAX_SHOTS; i++)
                    if (photons[i].active && missle.isColliding(photons[i]))
                    {
                        explode(missle);
                        stopMissle();
                        score += MISSLE_POINTS;
                    }
                if (missle.active && ship.active &&
                    hyperCounter <= 0 && ship.isColliding(missle))
                {
                    //System.out.println("ship collision");

                    explode(ship);
                    stopShip();
                    stopUfo();
                    stopMissle();

                }
                //System.out.println("");
            }
        }
    }

    public void guideMissle()
    {
        double dx, dy, angle;

        if (!ship.active || hyperCounter > 0)
            return;

        // Find the angle needed to hit the ship.

        dx = ship.x - missle.x;
        dy = ship.y - missle.y;
        if (dx == 0 && dy == 0)
            angle = 0;
        if (dx == 0)
        {
            if (dy < 0)
                angle = -Math.PI / 2;
            else
                angle = Math.PI / 2;
        }
        else
        {
            angle = Math.Atan(Math.Abs(dy / dx));
            if (dy > 0)
                angle = -angle;
            if (dx < 0)
                angle = Math.PI - angle;
        }

        // Adjust angle for screen coordinates.
        missle.angle = angle - Math.PI / 2;

        // Change the missle's angle so that it points toward the ship.
        missle.deltaX = 0.75 * MAX_ROCK_SPEED * -Math.Sin(missle.angle);
        missle.deltaY = 0.75 * MAX_ROCK_SPEED * Math.Cos(missle.angle);
    }

    public void stopMissle()
    {
        missle.active = false;
        missleCounter = 0;
    }

    public void initAsteroids()
    {
        int i, j;
        int s;
        double theta, r;
        int x, y;

        // Create random shapes, positions and movements for each asteroid.

        for (i = 0; i < MAX_ROCKS; i++)
        {

            // Create a jagged shape for the asteroid and give it a random rotation.

            asteroidss[i].shape = new myShape();
            s = MIN_ROCK_SIDES + (int)(random.NextDouble() * (MAX_ROCK_SIDES - MIN_ROCK_SIDES));
            for (j = 0; j < s; j++)
            {
                theta = 2 * Math.PI / s * j;
                r = MIN_ROCK_SIZE + (int)(random.NextDouble() * (MAX_ROCK_SIZE - MIN_ROCK_SIZE));
                x = (int)-Math.Round(r * Math.Sin(theta));
                y = (int)Math.Round(r * Math.Cos(theta));
                asteroidss[i].shape.addPoint(x, y);
            }
            asteroidss[i].active = true;
            asteroidss[i].angle = 0.0;
            asteroidss[i].deltaAngle = random.NextDouble() * 2 * MAX_ROCK_SPIN - MAX_ROCK_SPIN;

            // Place the asteroid at one edge of the screen.

            if (random.NextDouble() < 0.5)
            {
                asteroidss[i].x = -AsteroidObject.width / 2;
                if (random.NextDouble() < 0.5)
                    asteroidss[i].x = AsteroidObject.width / 2;
                asteroidss[i].y = random.NextDouble() * AsteroidObject.height;
            }
            else
            {
                asteroidss[i].x = random.NextDouble() * AsteroidObject.width;
                asteroidss[i].y = -AsteroidObject.height / 2;
                if (random.NextDouble() < 0.5)
                    asteroidss[i].y = AsteroidObject.height / 2;
            }

            // Set a random motion for the asteroid.

            asteroidss[i].deltaX = random.NextDouble() * asteroidsSpeed;
            if (random.NextDouble() < 0.5)
                asteroidss[i].deltaX = -asteroidss[i].deltaX;
            asteroidss[i].deltaY = random.NextDouble() * asteroidsSpeed;
            if (random.NextDouble() < 0.5)
                asteroidss[i].deltaY = -asteroidss[i].deltaY;

            asteroidss[i].render();
            asteroidIsSmall[i] = false;
        }

        asteroidsCounter = STORM_PAUSE;
        asteroidsLeft = MAX_ROCKS;
        if (asteroidsSpeed < MAX_ROCK_SPEED)
            asteroidsSpeed += 0.5;
    }

    public void initSmallAsteroids(int n)
    {
        int count;
        int i, j;
        int s;
        double tempX, tempY;
        double theta, r;
        int x, y;

        // Create one or two smaller asteroids from a larger one using inactive
        // asteroids. The new asteroids will be placed in the same position as the
        // old one but will have a new, smaller shape and new, randomly generated
        // movements.

        count = 0;
        i = 0;
        tempX = asteroidss[n].x;
        tempY = asteroidss[n].y;
        do
        {
            if (!asteroidss[i].active)
            {
                asteroidss[i].shape = new myShape();
                s = MIN_ROCK_SIDES + (int)(random.NextDouble() * (MAX_ROCK_SIDES - MIN_ROCK_SIDES));
                for (j = 0; j < s; j++)
                {
                    theta = 2 * Math.PI / s * j;
                    r = (MIN_ROCK_SIZE + (int)(random.NextDouble() * (MAX_ROCK_SIZE - MIN_ROCK_SIZE))) / 2;
                    x = (int)-Math.Round(r * Math.Sin(theta));
                    y = (int)Math.Round(r * Math.Cos(theta));
                    asteroidss[i].shape.addPoint(x, y);
                }
                asteroidss[i].active = true;
                asteroidss[i].angle = 0.0;
                asteroidss[i].deltaAngle = random.NextDouble() * 2 * MAX_ROCK_SPIN - MAX_ROCK_SPIN;
                asteroidss[i].x = tempX;
                asteroidss[i].y = tempY;
                asteroidss[i].deltaX = random.NextDouble() * 2 * asteroidsSpeed - asteroidsSpeed;
                asteroidss[i].deltaY = random.NextDouble() * 2 * asteroidsSpeed - asteroidsSpeed;
                asteroidss[i].render();
                asteroidIsSmall[i] = true;
                count++;
                asteroidsLeft++;
            }
            i++;
        } while (i < MAX_ROCKS && count < 2);

    }

    public void updateAsteroids()
    {
        int i, j;

        // Move any active asteroids and check for collisions.
        for (i = 0; i < MAX_ROCKS; i++)
            if (asteroidss[i].active)
            {
                asteroidss[i].advance();
                asteroidss[i].render();

                // If hit by photon, kill asteroid and advance score. If asteroid is
                // large, make some smaller ones to replace it.
                for (j = 0; j < MAX_SHOTS; j++)
                    if (photons[j].active && asteroidss[i].active && asteroidss[i].isColliding(photons[j]))
                    {
                        asteroidsLeft--;
                        asteroidss[i].active = false;
                        photons[j].active = false;
                        explode(asteroidss[i]);
                        if (!asteroidIsSmall[i])
                        {
                            score += BIG_POINTS;
                            initSmallAsteroids(i);
                        }
                        else
                            score += SMALL_POINTS;
                    }

                // If the ship is not in hyperspace, see if it is hit.

                if (ship.active && hyperCounter <= 0 &&
                    asteroidss[i].active && asteroidss[i].isColliding(ship))
                //if (ship.active && hyperCounter <= 0 &&
                //		asteroids[i].active && ship.isColliding(asteroids[i])) 
                {
                    //System.out.println("ship collision2");
                    vibrateDelay = 0;

                    explode(ship);
                    stopShip();
                    stopUfo();
                    stopMissle();

                }
                //System.out.println("");



            }
    }

    public void initExplosions()
    {
        int i;

        for (i = 0; i < MAX_SCRAP; i++)
        {
            explosions[i].shape = new myShape();
            explosions[i].active = false;
            explosionCounter[i] = 0;
        }
        explosionIndex = 0;
    }

    public void explode(AsteroidObject s)
    {
        int c, i, j;
        int cx, cy;

        // Create sprites for explosion animation. The each individual line segment
        // of the given sprite is used to create a new sprite that will move
        // outward  from the sprite's original position with a random rotation.

        s.render();
        c = 2;
        if (detail || s.sprite.npoints < 6)
            c = 1;
        for (i = 0; i < s.sprite.npoints; i += c)
        {
            explosionIndex++;
            if (explosionIndex >= MAX_SCRAP)
                explosionIndex = 0;
            explosions[explosionIndex].active = true;
            explosions[explosionIndex].shape = new myShape();
            j = i + 1;
            if (j >= s.sprite.npoints)
                j -= s.sprite.npoints;
            cx = (int)((s.shape.xpoints[i] + s.shape.xpoints[j]) / 2);
            cy = (int)((s.shape.ypoints[i] + s.shape.ypoints[j]) / 2);
            explosions[explosionIndex].shape.addPoint(
                s.shape.xpoints[i] - cx,
                s.shape.ypoints[i] - cy);
            explosions[explosionIndex].shape.addPoint(
                s.shape.xpoints[j] - cx,
                s.shape.ypoints[j] - cy);
            explosions[explosionIndex].x = s.x + cx;
            explosions[explosionIndex].y = s.y + cy;
            explosions[explosionIndex].angle = s.angle;
            explosions[explosionIndex].deltaAngle = 4 * (random.NextDouble() * 2 * MAX_ROCK_SPIN - MAX_ROCK_SPIN);
            explosions[explosionIndex].deltaX = (random.NextDouble() * 2 * MAX_ROCK_SPEED - MAX_ROCK_SPEED + s.deltaX) / 2;
            explosions[explosionIndex].deltaY = (random.NextDouble() * 2 * MAX_ROCK_SPEED - MAX_ROCK_SPEED + s.deltaY) / 2;
            explosionCounter[explosionIndex] = SCRAP_COUNT;
        }
    }

    public void updateExplosions()
    {
        int i;

        // Move any active explosion debris. Stop explosion when its counter has
        // expired.

        for (i = 0; i < MAX_SCRAP; i++)
            if (explosions[i].active)
            {
                explosions[i].advance();
                explosions[i].render();
                if (--explosionCounter[i] < 0)
                    explosions[i].active = false;
            }
    }


    public void ControllerCheck()
    {
        // Check if any cursor keys where released and set flags.
        left    = false;
        right   = false;
        up      = false;
        down    = false;

        if (!GamePad.GetState(PlayerIndex.One).IsConnected)
        {
            return;
        }


        shipDir = new Vector2(0, 0);

        float leftVal  = GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.X;
        float rightVal = GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.Y;

        // Check if any cursor keys have been pressed and set flags.
        if (SHIP_DIR_CONTROL)
        {
            if (Math.Abs(leftVal)  > 0.0001f ||
                Math.Abs(rightVal) > 0.0001f )
            {
                shipDir = new Vector2(leftVal,
                                      -rightVal);
            }
        }
        else
        {
            float threshHold = 0.5f;
            if (GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.X < -threshHold)
                left = true;
            if (GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.X > threshHold)
                right = true;
            if (GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.Y < -threshHold)
                down = true;
            if (GamePad.GetState(PlayerIndex.One).ThumbSticks.Left.Y > threshHold)
                up = true;
        }
            

        // Fire a photon and start its counter.
        photonTime += gameTime.ElapsedGameTime.Milliseconds;

        if ((GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed) && ship.active && !paused)
        {
            if (photonTime > FIRE_DELAY)
            {
                photonTime = 0;
                photonIndex++;
                if (photonIndex >= MAX_SHOTS)
                    photonIndex = 0;
                photons[photonIndex].active = true;
                photons[photonIndex].x = ship.x;
                photons[photonIndex].y = ship.y;
                photons[photonIndex].deltaX = 2 * MAX_ROCK_SPEED * -Math.Sin(ship.angle);
                photons[photonIndex].deltaY = 2 * MAX_ROCK_SPEED * Math.Cos(ship.angle);

                soundBank.PlayCue("fire");
            }
        }

  


        if (playing)
        {
            if (ufo.active)
            {
                soundBank.PlayCue("alien");
            }
        }

        // Warp ship into hyperspace by moving to a random location and
        // starting counter.

        if ((GamePad.GetState(PlayerIndex.One).Buttons.Y == ButtonState.Pressed) && ship.active && hyperCounter <= 0 && !paused)
        {
            ship.x = random.NextDouble() * AsteroidObject.width;
            ship.y = random.NextDouble() * AsteroidObject.height;
            hyperCounter = HYPER_COUNT;
        }
        // Toggle graphics detail on or off.

        // Won't bother with the detail button for this release on xbox!
        //if (GamePad.GetState(PlayerIndex.One).Buttons.X == ButtonState.Pressed)
        //    detail = !detail;

        pauseDelay--;
        if (pauseDelay<0) pauseDelay=0;
        if (playing)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Start == ButtonState.Pressed)
            {
                if (pauseDelay<=0)
                {
                    pauseDelay=50;
                    paused = !paused;
                }
            }
        }

        bool aButton = (GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed);
        bool sButton = (GamePad.GetState(PlayerIndex.One).Buttons.Start == ButtonState.Pressed);
        // 'S' key: start the game, if not already in progress.
        if ( (aButton || sButton) && !playing )
        {
            pauseDelay=50;
            initGame();
        }
    }

    protected override void Initialize()
    {
        audioEngine = new AudioEngine("Content/sounds.xgs");
        waveBank = new WaveBank(audioEngine, "Content/myWaveBank.xwb");
        soundBank = new SoundBank(audioEngine, "Content/mySoundBank.xsb");

        base.Initialize();
    }

    protected override void LoadContent()
    {
        
        // Create a new SpriteBatch, which can be used to draw textures.
        SpriteFont spriteFont = Content.Load<SpriteFont>("myFont");
        offGraphics = new myGraphics(graphics.GraphicsDevice, spriteFont);

        GraphicsDevice device = graphics.GraphicsDevice;

        myVertexDeclaration = new VertexDeclaration(graphics.GraphicsDevice, VertexPositionColor.VertexElements);

        // Create the effect that will be used to draw the axis
        effect = new BasicEffect(device, null);
        // Calculate the effect aspect ratio, projection, and view matrix
        float aspectRatio = (float)device.Viewport.Width / device.Viewport.Height;
        effect.View = Matrix.CreateLookAt(new Vector3(0.0f, 0.0f, 50.0f), 
                                          Vector3.Zero,
                                          Vector3.Up);
        effect.World = Matrix.Identity;
        effect.Projection = Matrix.CreatePerspectiveFieldOfView(
        MathHelper.ToRadians(45.0f), aspectRatio, 1.0f, 500.0f);
        effect.LightingEnabled = false;

        base.LoadContent();
        init();

    }

    protected override void Update(GameTime gameTime)
    {
        GamerServicesDispatcher.Update();
        if (Guide.IsVisible)
        {
            return;
        }

        constrollerDisconnected = false;
        if (!GamePad.GetState(PlayerIndex.One).IsConnected)
        {
            constrollerDisconnected = true;
            return;
        }


        audioEngine.Update();

        // Allows the game to exit
        if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
            this.Exit();

        this.gameTime = gameTime;

        run();

        ControllerCheck();

        base.Update(gameTime);
    }

    protected override void Draw(GameTime gameTime)
    {
        graphics.GraphicsDevice.Clear(Color.Black);

        // Debug draw the safe screen!
        //offGraphics.DrawSafeScreen();

        graphics.GraphicsDevice.VertexDeclaration = myVertexDeclaration;

        effect.LightingEnabled = false;
        effect.VertexColorEnabled = true;
        effect.Begin();
        foreach (EffectPass pass in effect.CurrentTechnique.Passes)
        {
            
            pass.Begin();
            drawGame();
            pass.End();
        }
        effect.End();

        drawGameText();
  
        base.Draw(gameTime);
    }


    protected void drawGame()
    {
        /*
        {
            Vector2[] pnts = new Vector2[3];
            pnts[0] = new Vector2(579, 327);
            pnts[1] = new Vector2(595, 341);
            pnts[2] = new Vector2(583, 348);
            for (int ii = 0; ii < 3; ii++)
            {
                offGraphics.drawLine((int)pnts[ii].X, (int)pnts[ii].Y, (int)pnts[(ii + 1) % 3].X, (int)pnts[(ii + 1) % 3].Y);
            }

            offGraphics.drawLine(2000, 2000, 376, 88);
            
            
            int[] xx = new int[3];
            int[] yy = new int[3];
            for (int ii=0; ii<3; ii++)
            {
                xx[ii] = (int)pnts[ii].X;
                yy[ii] = (int)pnts[ii].Y;
            }

            myShape testShape = new myShape();
            if (testShape.insidePoly(xx, yy, 3, 376, 88))
            {
                int www = 2;
            }

            string ssss = "";
        }
         * */


        int w = graphics.GraphicsDevice.Viewport.Width;
        int h = graphics.GraphicsDevice.Viewport.Height;

        /*
        Dimension d = getSize();
        int i;
        int c;
        String s;
        int w, h;
        int x, y;

        // Create the off screen graphics context, if no good one exists.
        if (offGraphics == null || d.width != offDimension.width || d.height != offDimension.height)
        {
            offDimension = d;
            offImage = createImage(d.width, d.height);
            offGraphics = offImage.getGraphics();
        }
        */

        // Fill in background and stars.
        offGraphics.setColor(Color.Black);
        offGraphics.fillRect(0, 0, w, h);
        if (detail)
        {
            offGraphics.setColor(Color.White);
            for (int i = 0; i < numStars; i++)
                offGraphics.drawLine(stars[i].X, stars[i].Y, stars[i].X + 1, stars[i].Y + 1);
        }

        // Draw photon bullets.
        offGraphics.setColor(Color.Yellow);
        for (int i = 0; i < MAX_SHOTS; i++)
            if (photons[i].active)
            {
                photons[i].drawPolyline(offGraphics, offGraphics.getColor());
            }



        // Draw the guided missle, counter is used to quickly fade color to black
        // when near expiration.
        int c = Math.Min(missleCounter * 24, 255);
        offGraphics.setColor(new Color((byte)c, (byte)c, (byte)c));
        if (missle.active)
        {
            offGraphics.setColor(Color.YellowGreen);

            missle.drawPolyline(offGraphics, offGraphics.getColor());

            offGraphics.drawLine(missle.sprite.xpoints[missle.sprite.npoints - 1],
                                 missle.sprite.ypoints[missle.sprite.npoints - 1],
                                 missle.sprite.xpoints[0], missle.sprite.ypoints[0]);
        }


        // Draw the asteroids.

        for (int i = 0; i < MAX_ROCKS; i++)
            if (asteroidss[i].active)
            {
                if (detail)
                {

                    //offGraphics.setColor(Color.black);

                    //// Draws a sequence of connected lines defined by arrays of x and y coordinates.
                    //offGraphics.fillPolygon ( asteroids[i].sprite.xpoints, // int[] x points
                    //                          asteroids[i].sprite.ypoints, // int[] y points
                    //                          asteroids[i].sprite.npoints);// number points
                    //                          
                    asteroidss[i].fillPolygon(offGraphics, Color.Black);

                }

                //offGraphics.setColor(Color.white);

                //// Draws a sequence of connected lines defined by arrays of x and y coordinates.
                //offGraphics.drawPolyline( asteroids[i].sprite.xpoints, // int[] x points
                //                          asteroids[i].sprite.ypoints, // int[] y points
                //                          asteroids[i].sprite.npoints);// number points

                asteroidss[i].drawPolyline(offGraphics, Color.White);

                offGraphics.drawLine(asteroidss[i].sprite.xpoints[asteroidss[i].sprite.npoints - 1], asteroidss[i].sprite.ypoints[asteroidss[i].sprite.npoints - 1],
                                    asteroidss[i].sprite.xpoints[0], asteroidss[i].sprite.ypoints[0]);
            }


        // Draw the flying saucer.
        if (ufo.active)
        {
            if (detail)
            {
                offGraphics.setColor(Color.Black);

                // Draws a sequence of connected lines defined by arrays of x and y coordinates.
                offGraphics.fillPolygon(ufo.sprite.xpoints, // int[] x points
                                        ufo.sprite.ypoints, // int[] y points
                                        ufo.sprite.npoints);// number points
            }

            ufo.drawPolyline(offGraphics, Color.Violet);

            offGraphics.drawLine(ufo.sprite.xpoints[ufo.sprite.npoints - 1], ufo.sprite.ypoints[ufo.sprite.npoints - 1],
                                ufo.sprite.xpoints[0], ufo.sprite.ypoints[0]);
        }//End if(ufo..


        // Draw the ship, counter is used to fade color to white on hyperspace.

        c = 255 - (255 / HYPER_COUNT) * hyperCounter;
        if (ship.active)
        {
            if (detail && hyperCounter == 0)
            {
                ship.drawPolyline(offGraphics, Color.Black);
            }

            ship.drawPolyline(offGraphics, new Color((byte)c, (byte)c, (byte)c));

            offGraphics.drawLine(ship.sprite.xpoints[ship.sprite.npoints - 1], ship.sprite.ypoints[ship.sprite.npoints - 1],
                                ship.sprite.xpoints[0], ship.sprite.ypoints[0]);

            // Draw thruster exhaust if thrusters are on. Do it randomly to get a
            // flicker effect.

            if (!paused && detail && random.NextDouble() < 0.5)
            {

                if (SHIP_DIR_CONTROL)
                {
                    //double moveSpeed = Math.Sqrt(ship.deltaX * ship.deltaX + ship.deltaY * ship.deltaY);
                    if (Math.Abs(shipDir.X) > 0.1f || Math.Abs(shipDir.Y) > 0.1f)
                    {
                        {
                            fwdThruster.drawPolyline(offGraphics, offGraphics.getColor());

                            offGraphics.drawLine(fwdThruster.sprite.xpoints[fwdThruster.sprite.npoints - 1], fwdThruster.sprite.ypoints[fwdThruster.sprite.npoints - 1],
                                                 fwdThruster.sprite.xpoints[0], fwdThruster.sprite.ypoints[0]);
                        }

                        /*
                        {
                            revThruster.drawPolyline(offGraphics, offGraphics.getColor());

                            offGraphics.drawLine(revThruster.sprite.xpoints[revThruster.sprite.npoints - 1], revThruster.sprite.ypoints[revThruster.sprite.npoints - 1],
                                            revThruster.sprite.xpoints[0], revThruster.sprite.ypoints[0]);
                        }
                        */
                    }
                }

                if (up)
                {
                    fwdThruster.drawPolyline(offGraphics, offGraphics.getColor());

                    offGraphics.drawLine(fwdThruster.sprite.xpoints[fwdThruster.sprite.npoints - 1], fwdThruster.sprite.ypoints[fwdThruster.sprite.npoints - 1],
                                         fwdThruster.sprite.xpoints[0], fwdThruster.sprite.ypoints[0]);
                }
                if (down)
                {
                    revThruster.drawPolyline(offGraphics, offGraphics.getColor());

                    offGraphics.drawLine(revThruster.sprite.xpoints[revThruster.sprite.npoints - 1], revThruster.sprite.ypoints[revThruster.sprite.npoints - 1],
                                    revThruster.sprite.xpoints[0], revThruster.sprite.ypoints[0]);
                }
            }
        }

        // Draw any explosion debris, counters are used to fade color to black.
        for (int i = 0; i < MAX_SCRAP; i++)
            if (explosions[i].active)
            {
                c = (255 / SCRAP_COUNT) * explosionCounter[i];

                explosions[i].drawPolyline(offGraphics, new Color((byte)c, (byte)c, (byte)c));
            }


        /*
        // ** DEBUGGING ** //
        for (int i = 0; i < MAX_ROCKS; i++)
        //if (ship.active && asteroidss[i].active)
        {
            ship.RenderisCollision(offGraphics, asteroidss[i]);

        }
        */

    }



    protected void drawGameText()
    {
        int height = graphics.GraphicsDevice.Viewport.Height;
        int width = graphics.GraphicsDevice.Viewport.Width;

        //offGraphics.setFont(font);
        //offGraphics.setColor(Color.white);

        int fontHeight = (int)offGraphics.spriteFont.MeasureString("height").Y;

        offGraphics.setColor(Color.Tomato);
        
        string s = "";
        offGraphics.drawString("Score: " + score,     offGraphics.SAFE_SCREEN_LEFT, offGraphics.SAFE_SCREEN_TOP);
        offGraphics.drawString("Ships: " + shipsLeft, offGraphics.SAFE_SCREEN_LEFT, offGraphics.SAFE_SCREEN_BOTTOM - fontHeight);
       
        s = "High: " + highScore;
        int maxHightWidth = (int)offGraphics.spriteFont.MeasureString(s).X;
        offGraphics.drawString(s, offGraphics.SAFE_SCREEN_RIGHT-maxHightWidth, offGraphics.SAFE_SCREEN_TOP);

        if (constrollerDisconnected)
        {
            s = "Controller Disconnected";
            offGraphics.drawString(s, (int)(width - offGraphics.spriteFont.MeasureString(s).X) / 2, height / 4);
        }

        int y_pos_text = height / 2;
        if (!playing)
        {
            s = szName;
            offGraphics.drawString(s, (int)(width - offGraphics.spriteFont.MeasureString(s).X) / 2, y_pos_text);
            y_pos_text += fontHeight;
            s = szVers;
            offGraphics.drawString(s, (int)(width - offGraphics.spriteFont.MeasureString(s).X) / 2, y_pos_text);

            gameOverDisplayDelay++;
            if (gameOverDisplayDelay < 200)
            {
                y_pos_text += fontHeight;
                s = "Game Over";
                offGraphics.drawString(s, (int)(width - offGraphics.spriteFont.MeasureString(s).X) / 2, y_pos_text);
            }
            else
            {
                gameOverDisplayDelay = 10000;
                y_pos_text += fontHeight;
                s = "Press Start To Play";
                offGraphics.drawString(s, (int)(width - offGraphics.spriteFont.MeasureString(s).X) / 2, height / 4 + y_pos_text);

                y_pos_text += fontHeight;
                s = "Press Back To Exit";
                offGraphics.drawString(s, (int)(width - offGraphics.spriteFont.MeasureString(s).X) / 2, height / 4 + y_pos_text);
            }

        }
        else if (paused)
        {
            gameOverDisplayDelay = 0;
            s = "Game Paused";
            offGraphics.drawString(s, (int)(width - offGraphics.spriteFont.MeasureString(s).X) / 2, height / 4);
        }
    }

}//End asteroid class





/*****************************************************************************/
/*                                                                           */
/*                       **PROGRAM ENTRY POINT**                             */
/*                                                                           */
/*****************************************************************************/
static class Program
{
    static void Main(string[] args)
    {
        using (asteroids game = new asteroids())
        {
            game.Run();
        }
    }
}




